package model;

import rac.*;
import javax.swing.*;

/**
 * STUDENTS TO COMPLETE
 * Implements a postfix calculator using a stack to hold the operands 
 * in a postfix expression.
 * Students to complete the "TO DO" code.
 * Students are free to add any non-public fields, methods, inner classes, 
 * static nested classes, top-level (i.e. not nested) classes.
 */
public class RPNMachine implements IRPNCalc{
    
    /**
     * Stack to hold the operands in a postfix expression
     */
    private IRAContainer _stack;
    
    /**
     * Holds input and intermediate results.
     */
    private double _acc;
    
    /**
     * String representation of _acc
     */
    private String _accRep;
           
    /**
     * Abstract state of RPN machine.
     */
    private static abstract class AState {
        abstract void enterDigit(int digit);
        abstract void enterPoint();
        abstract void enterOp(IBinOp op);
        abstract void push();
    }
    
    private AState _state;  // current state of the RPN machine
    
    /**
     * The start state.
     */
    private AState _startState = new AState() {
        void enterDigit(int digit) {
            // starts accumulating the digit as long as it is not 0.
            // change state if needed.
            // TO DO
        }
        
        void enterPoint() {
            // add the decimal point.
            // change state.
            // TO DO
        }
        
        void enterOp(IBinOp op) {
            try {
            // perform computation
            // change state.
            // TO DO
            }
            catch (Exception e) {
                _accRep = "Error!";
                _state = _errorState;
            }
        }
        
        void push() {
            // TO DO
        }
    };
    
    /**
     * State to accumulate the digits.
     */
    private AState _accumState = new AState() {
        void enterDigit(int digit) {
            // keep accumulating the digits
            // TO DO
        }
        
        void enterPoint() {
            // add the decimal point.
            // change state.
            // TO DO
        }
        
        void enterOp(IBinOp op) {
            try {
            // perform computation
            // change state.
            // TO DO
            }
            catch (Exception e) {
                _accRep = "Error!";
                _state = _errorState;
            }
        }
        
        void push() {
            _stack.put(new Double(_acc));
            _state = _startState;
        }
        
    };
    
    /**
     * State after a decimal point is entered.
     */
    private AState _pointState = new AState() {
        void enterDigit(int digit) {
            // Accumulate digit
            // TO DO
        }
        
        void enterPoint() {
            // TO DO
        }
        
        void enterOp(IBinOp op) {
            // TO DO;
        }
        
        void push() {
            // TO DO
        }
    };
    
    /**
     * State after an operation is entered.
     */
    private AState _compState = new AState() {
        void enterDigit(int digit) {
            _startState.enterDigit(digit);
        }
        
        void enterPoint() {
            _startState.enterPoint();
        }
        
        void enterOp(IBinOp op) {
            try {
                // Perform computation
                // TO DO
            }
            catch (Exception e) {
                _accRep = "Error!";
                _state = _errorState;
            }
        }
        
        void push() {
            _stack.put(new Double(_acc));
            _state = _startState;
        }
    };
    
    /**
     * Do nothing state: Null Object pattern!
     */
    private AState _errorState = new AState() {
        void enterDigit(int digit) {
            // no-op
        }
        
        void enterPoint() {
            // no-op
        }
        
        void enterOp(IBinOp op) {
            // no op
        }
        
        void push() {
            // no-op
        }
    };
    
    
    public RPNMachine() {
        _stack = new LRSStackFactory().makeRAC();
        clear();
    }
    
    // It's the state pattern: delegate state-dependent behavior to the state!
    public String enterDigit(int d) {
        _state.enterDigit(d);
        return _accRep;
    }
    
    public String enterPoint() {
        _state.enterPoint();
        return _accRep;
    }
    
    public String enterOp(IBinOp op) {
        _state.enterOp(op);
        return _accRep;
    }
    
    public String push() {
        _state.push();
        return _accRep;
    }
          
    /**
     * Clear is state-independent!
     */
    public String clear() {
        _stack.clear();
        _state = _startState;
        _acc = 0;
        _accRep = "0";
        return _accRep;
    }
}
