package view;

import model.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;

/**
 * View to display an RPN calculator.
 */
public class RPNView extends JFrame {
    
    private JPanel _buttonsPnl = new JPanel();
    private JButton[] _numBtns = new JButton[10];
    private JButton _pointBtn = new JButton(".");
    private JButton _plusBtn = new JButton("+");
    private JButton _timesBtn = new JButton("*");
    private JButton _pushBtn = new JButton("Push");
    private JButton _clearBtn = new JButton("Clear");
    private JPanel _topPnl = new JPanel();
    private JLabel _accDummyLbl = new JLabel("Accumulator: ");
    private JLabel _accLbl = new JLabel("0");
    
    /**
     * The calculator model.
     */
    private IRPNCalc _rpnMachine;
    
    public RPNView() {
        _rpnMachine = new RPNMachine();
        initGui();
    }
    
    private void initGui() {
        getContentPane().setLayout(new BorderLayout(3,3));
        _topPnl.setLayout(new BoxLayout(_topPnl, BoxLayout.X_AXIS));
        _topPnl.add(_accDummyLbl);
        _topPnl.add(_accLbl);
        _accLbl.setHorizontalAlignment(SwingConstants.RIGHT);
        _accLbl.setHorizontalTextPosition(SwingConstants.RIGHT);
        
        GridLayout glm = new GridLayout(6,3,2,2);
        _buttonsPnl = new JPanel();
        _buttonsPnl.setLayout(glm);
        for(int i=9; i>=0; i--) {
            _numBtns[i] = new JButton(String.valueOf(i));
            _buttonsPnl.add(_numBtns[i]);
            final int j = i;
            _numBtns[i].addActionListener(new ActionListener () {
                public void actionPerformed(ActionEvent e) {
                    updateDisplay(_rpnMachine.enterDigit(j));
                }
            });
        }
        _buttonsPnl.add(_pointBtn);
        _buttonsPnl.add(_plusBtn);
        _buttonsPnl.add(_timesBtn);
        _buttonsPnl.add(_pushBtn);
        _buttonsPnl.add(_clearBtn);
        
        _pointBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                updateDisplay(_rpnMachine.enterPoint());
            }
        });
        
        _plusBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                updateDisplay(_rpnMachine.enterOp(AddOp.Singleton));
            }
        });
        
        _timesBtn.addActionListener(new ActionListener() {
            public void actionPerformed (ActionEvent e) {
                updateDisplay(_rpnMachine.enterOp(MulOp.Singleton));
            }
        });
        
        _pushBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                updateDisplay(_rpnMachine.push());
            }
        });
        
        _clearBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                updateDisplay(_rpnMachine.clear());
            }
            
        });
        
        getContentPane().add(_buttonsPnl, BorderLayout.CENTER);
        getContentPane().add(_topPnl, BorderLayout.NORTH);
        pack();
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setVisible(true);
    }
    
    private void updateDisplay(String s) {
        _accLbl.setText(s);
    }
    
    public static void main (String[] args) {
        new RPNView();
    }
}
